| package |
package := Package name: 'TaskbarIcon'.
package paxVersion: 0;
	basicComment: ''.

package basicScriptAt: #postinstall put: '"define new fields on NOTIFYICONDATA"
NOTIFYICONDATA defineFields.'.
package basicScriptAt: #preinstall put: '"Add new constants to win32 pool for ShellLibrary version >= 5"
Win32Constants
	"messages"
	add: ''NIM_SETVERSION''->4;
	add: ''NIM_SETFOCUS''->3;
	"balloon tooltip"
	add: ''NIF_INFO''->16;
	add: ''NIIF_ERROR''->3;
	add: ''NIIF_INFO''->1;
	add: ''NIIF_NONE''->0;
	add: ''NIIF_WARNING''->2;
	add: ''NIIF_ICON_MASK''->15;
	add: ''NIIF_NOSOUND''->16;
	"uVersion"
	add: ''NOTIFYICON_VERSION''->3;
	"dwState and dwStateMask"
	add: ''NIF_STATE''->8;
	add: ''NIS_HIDDEN''->1;
	add: ''NIS_SHAREDICON''->2;
	"Notification Interaction. These are offset by WM_USER"
	add: ''NIN_BALLOONSHOW'' ->2;
	add: ''NIN_BALLOONHIDE'' -> 3;
	add: ''NIN_BALLOONTIMEOUT'' -> 4;
	add: ''NIN_BALLOONUSERCLICK'' -> 5;'.

package classNames
	add: #TaskbarIcon;
	add: #TaskbarMenu;
	add: #TaskbarPresenterAbstract;
	yourself.

package methodNames
	add: #NOTIFYICONDATA -> #notify:title:for:withFlags:;
	add: #NOTIFYICONDATA -> #tipText:;
	add: 'NOTIFYICONDATA class' -> #defineFields;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	yourself).

package!

"Class Definitions"!

Menu subclass: #TaskbarMenu
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Presenter subclass: #TaskbarPresenterAbstract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
View subclass: #TaskbarIcon
	instanceVariableNames: 'pnid icon'
	classVariableNames: 'TaskbarMessage TaskbarMessageMap'
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!NOTIFYICONDATA methodsFor!

notify: aText title: aTitleText for: aIntMillisecs withFlags: aNIIFFlags
	self szInfo: aText; szInfoTitle: aTitleText; uTimeout: aIntMillisecs; dwInfoFlags: aNIIFFlags.
	self uFlags: (self uFlags bitOr: NIF_INFO).!

tipText: aString
	"Set the receiver's szTip (tip text) field."

	|tmpString|
	"ShellLibrary earlier than version 5 accepts tipText up to 64 characters in length"
	tmpString := ((ShellLibrary default versionInfo productMajor highWord < 5) and: [aString size > 64]) 
		ifTrue: [aString copyFrom: 1 to: 64] 
		ifFalse: [aString].
	self szTip: tmpString.
	self uFlags: (self uFlags bitOr: NIF_TIP)! !
!NOTIFYICONDATA categoriesFor: #notify:title:for:withFlags:!accessing!public! !
!NOTIFYICONDATA categoriesFor: #tipText:!accessing!public! !

!NOTIFYICONDATA class methodsFor!

defineFields
	"Define the fields of the NOTIFYICONDATA 'structure'.
		self compileDefinition

		struct {
			DWORD cbSize; 
			HWND hWnd; 
			UINT uID; 
			UINT uFlags; 
			UINT uCallbackMessage; 
			HICON hIcon; 
			// NOTE that in ShellLibrary version < 5.0  this is limited to 64
			char szTip[128]; 
			// Available in ShellLibrary version >= 5.0
		        DWORD dwState;
		        DWORD dwStateMask;
        			CHAR   szInfo[256];
        			union {
            			UINT  uTimeout;
            			UINT  uVersion;
        			} DUMMYUNIONNAME;
        			CHAR   szInfoTitle[64];
        			DWORD dwInfoFlags;
			// NOTE Available in ShellLibrary version >= 6
		        GUID guidItem;
		} NOTIFYICONDATA;
	"
 
	self
		defineField: #cbSize			type: DWORDField writeOnly;
		defineField: #hWnd			type: HANDLEField writeOnly;
		defineField: #uID			type: DWORDField writeOnly;
		defineField: #uFlags			type: DWORDField new;
		defineField: #uCallbackMessage	type: DWORDField writeOnly;
		defineField: #hIcon			type: HANDLEField writeOnly;
		defineField: #szTip			type: (StringField length: 128) beWriteOnly;
		defineField: #dwState			type: DWORDField new;
		defineField: #dwStateMask	type: DWORDField writeOnly;
		defineField: #szInfo			type: (StringField length: 256) beWriteOnly;
		defineField: #uTimeout		type: DWORDField writeOnly;
		defineField: #szInfoTitle		type: (StringField length: 64) beWriteOnly;
		defineField: #dwInfoFlags		type: DWORDField writeOnly;
		defineField: #guidItem		type: (StructureField type: GUID) beFiller.

! !
!NOTIFYICONDATA class categoriesFor: #defineFields!initializing!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

TaskbarMenu guid: (GUID fromString: '{68EC8264-076A-42AA-87E1-D73BD179B215}')!
TaskbarMenu comment: ''!
!TaskbarMenu categoriesForClass!Graphics-Tools! !
!TaskbarMenu methodsFor!

showIn: aView position: aPos
	"http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/winui/windowsuserinterface/resources/menus/menureference/menufunctions/trackpopupmenu.asp

To display a context menu for a notification icon, the current window must be the foreground window before the application calls TrackPopupMenu. Otherwise, the menu will not disappear when the user clicks outside of the menu or the window that created the menu (if it is visible). However, when the current window is the foreground window, the second time this menu is displayed, it displays and then immediately disappears. To correct this, you must force a task switch to the application that called TrackPopupMenu at some time in the near future. This is done by posting a benign message to the window or thread, as shown in the following code sample: 

   SetForegroundWindow(hDlg);
   TrackPopupMenu(   hSubMenu, TPM_RIGHTBUTTON,pt.x,pt.y, 0, hDlg, NULL);
   PostMessage(hDlg, WM_NULL, 0, 0);
"
	aView beForeground. "SetForegroundWindow"
	super showIn: aView position: aPos. "TrackPopupMenu"
	aView 
		postMessage: WM_NULL
		wParam: 0
		lParam: 0. "PostMessage"
! !
!TaskbarMenu categoriesFor: #showIn:position:!public! !

TaskbarPresenterAbstract guid: (GUID fromString: '{78E376D9-AC26-4316-BF81-6703B85524E5}')!
TaskbarPresenterAbstract comment: ''!
!TaskbarPresenterAbstract categoriesForClass!MVP-Presenters! !
!TaskbarPresenterAbstract methodsFor!

contextMenu
	^(TaskbarMenu fromStrings: self menuStrings)
		setDefault: 1;
		yourself!

createView: aResourceNameString

	self view: ((TaskbarIcon icon: self class icon tipText: self taskbarName) contextMenu: self contextMenu).

	^self view!

exit
	self view close.
!

menuStrings
	" 
		Something like this:

		^ #('TaskbarApplication'
			'Command1//command1' 
			'Command2//command2' 
			'-' 
			'Exit//exit'
		)	

		Look at Menu class>>fromStrings: for further details.
		All commands should be 'must not strip'!!
	"

	^ self subclassResponsibility!

showShell
	self view show!

taskbarName
	"Default tip text etc."
	 ^ self subclassResponsibility! !
!TaskbarPresenterAbstract categoriesFor: #contextMenu!accessing!private! !
!TaskbarPresenterAbstract categoriesFor: #createView:!private! !
!TaskbarPresenterAbstract categoriesFor: #exit!commands!must not strip!public! !
!TaskbarPresenterAbstract categoriesFor: #menuStrings!accessing!constants!private! !
!TaskbarPresenterAbstract categoriesFor: #showShell!operations!public! !
!TaskbarPresenterAbstract categoriesFor: #taskbarName!accessing!constants!private! !

TaskbarIcon guid: (GUID fromString: '{84570C96-32F6-417B-8F68-9ED4C0415EA2}')!
TaskbarIcon comment: ''!
!TaskbarIcon categoriesForClass!Unclassified! !
!TaskbarIcon methodsFor!

destroy
	self hide.
	super destroy.!

dispatchRegistered: registeredId wParam: wParam lParam: lParam
	"Private - Dispatch the window message. lParam actually can contain a wide set of WM messages. We re-dispatch baed on lParam if a system message and handle the taskbar specific ones"
	| pt |
	pt := Cursor position.
	^lParam < WM_USER
		ifTrue: [super dispatchMessage: lParam wParam: self asParameter lParam: pt asDword]
		ifFalse: [
			| selector |
			selector := TaskbarMessageMap at: lParam - WM_USER ifAbsent: [ ^ true ].
			self perform: selector 
		].!

hide
	self sendTaskbar: NIM_DELETE!

icon: aIcon
	"Check if icon has changed and icon is shown, then NIM_MODIFY. Need to store icon to avoid GC"
	icon := aIcon.
	self pnid icon: aIcon.
	self sendTaskbar: NIM_MODIFY.

!

initialize
	super initialize.
	self parentView: self class desktop.
!

notify: aText title: aTitle
	self notify: aText title: aTitle withFlags: NIIF_NONE.!

notify: aText title: aTitle withFlags: aNIIFFlags
	self pnid notify: aText title: aTitle for: self notifyTimeout withFlags: aNIIFFlags.
	self sendTaskbar: NIM_MODIFY
!

notifyError: aText title: aTitle
	self notify: aText title: aTitle withFlags: NIIF_ERROR.
!

notifyInfo: aText title: aTitle
	self notify: aText title: aTitle withFlags: NIIF_INFO.
!

notifyTimeout
	^10000!

notifyWarning: aText title: aTitle
	self notify: aText title: aTitle withFlags: NIIF_WARNING.
!

onBalloonClicked
	self presenter trigger: #balloonClicked.
	^true!

onBalloonHide
	self presenter trigger: #balloonHide.
	^true
!

onBalloonShow
	self pnid uFlags: (self pnid uFlags bitAnd: NIF_INFO bitInvert).
	self presenter trigger: #balloonShow.
	^true
!

onBalloonTimeout
	self presenter trigger: #balloonTimeout.
	^true
!

pnid
	pnid isNil ifTrue: [
		pnid := (NOTIFYICONDATA new); uID: 1; hWnd: self handle; message: self class taskbarMessage.
		"pnid icon: (Bitmap fromFile: 'beacon.bmp')"].
	^pnid.!

sendTaskbar: aMessage
	"aMessage is one of the NIM_* messages"
	ShellLibrary default shell_NotifyIcon: aMessage pnid: self pnid!

show
	self sendTaskbar: NIM_ADD.!

tipText: aText
	self pnid tipText: aText displayString.
	self sendTaskbar: NIM_MODIFY.
! !
!TaskbarIcon categoriesFor: #destroy!public! !
!TaskbarIcon categoriesFor: #dispatchRegistered:wParam:lParam:!private! !
!TaskbarIcon categoriesFor: #hide!public! !
!TaskbarIcon categoriesFor: #icon:!initializing!public! !
!TaskbarIcon categoriesFor: #initialize!initializing!public! !
!TaskbarIcon categoriesFor: #notify:title:!notification!public! !
!TaskbarIcon categoriesFor: #notify:title:withFlags:!notification!public! !
!TaskbarIcon categoriesFor: #notifyError:title:!notification!public! !
!TaskbarIcon categoriesFor: #notifyInfo:title:!notification!public! !
!TaskbarIcon categoriesFor: #notifyTimeout!private! !
!TaskbarIcon categoriesFor: #notifyWarning:title:!notification!public! !
!TaskbarIcon categoriesFor: #onBalloonClicked!initializing!public! !
!TaskbarIcon categoriesFor: #onBalloonHide!initializing!public! !
!TaskbarIcon categoriesFor: #onBalloonShow!initializing!public! !
!TaskbarIcon categoriesFor: #onBalloonTimeout!initializing!public! !
!TaskbarIcon categoriesFor: #pnid!private! !
!TaskbarIcon categoriesFor: #sendTaskbar:!private! !
!TaskbarIcon categoriesFor: #show!public! !
!TaskbarIcon categoriesFor: #tipText:!initializing!public! !

!TaskbarIcon class methodsFor!

icon: aIcon 
	^self new icon: aIcon!

icon: aIcon tipText: aString
	^self new icon: aIcon; tipText: aString!

initialize
	"Private - TaskbarIcon initialize"
	TaskbarMessageMap:= LookupTable new.
	TaskbarMessageMap 
		at: WM_LBUTTONDOWN 			put: #onLeftButtonPressed;
		at: WM_RBUTTONDOWN 			put: #onRightButtonPressed;
		at: WM_LBUTTONDBLCLK 			put: #onLeftButtonDoubleClicked;
		at: WM_RBUTTONDBLCLK 			put: #onRightButtonDoubleClicked;
		at: WM_CONTEXTMENU			put: #onContextMenu;
		at: NIN_BALLOONSHOW 			put: #onBalloonShow;
		at: NIN_BALLOONHIDE 			put: #onBalloonHide;
		at: NIN_BALLOONTIMEOUT			put: #onBalloonTimeout;
		at: NIN_BALLOONUSERCLICK put: #onBalloonClicked.!

taskbarMessage
	TaskbarMessage isNil ifTrue: [
		TaskbarMessage := self registerMessage: 'TaskbarMessage'].
	^TaskbarMessage! !
!TaskbarIcon class categoriesFor: #icon:!instance creation!public! !
!TaskbarIcon class categoriesFor: #icon:tipText:!instance creation!public! !
!TaskbarIcon class categoriesFor: #initialize!private! !
!TaskbarIcon class categoriesFor: #taskbarMessage!public! !

"Binary Globals"!

"Resources"!

